/* 
-----------------------------------------------------------------------------
File  : dba.groovy
Author: Karl Bishop (kfbishop@us.ibm.com)
-----------------------------------------------------------------------------
(C) Copyright IBM Corp. 2010 All rights reserved.

You may only copy, modify, and distribute these samples internally.
These samples have not been tested under all conditions and are provided
to you by IBM without obligation of support of any kind.

IBM PROVIDES THESE SAMPLES "AS IS" SUBJECT TO ANY STATUTORY WARRANTIES THAT
CANNOT BE EXCLUDED. IBM MAKES NO WARRANTIES OR CONDITIONS, EITHER EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OR
CONDITIONS OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND
NON-INFRINGEMENT REGARDING THESE SAMPLES OR TECHNICAL SUPPORT, IF ANY.
-----------------------------------------------------------------------------
*/

import java.sql.Connection
import java.sql.DatabaseMetaData
import java.sql.Statement
import java.sql.ResultSet
import java.sql.ResultSetMetaData
import java.sql.Types

import zero.data.groovy.Manager


//---------------------------------------------------------------------------
// Return a list of DataSources defined for this application
//---------------------------------------------------------------------------
def getDataSources() {
    def dbs = []
    zlist("/config/db", true).each {
        dbs << (it - '/config/db/')
    }
    return dbs
}

//---------------------------------------------------------------------------
// Return information for a given DB
//---------------------------------------------------------------------------
def getDbInfo( dbId ) {
    def manager = Manager.create( dbId )
    DatabaseMetaData dbMetaData = manager.retrieveConnection().getMetaData();
    def map = [
        'db_url'         : dbMetaData.getURL(),
        'db_name'        : dbMetaData.getDatabaseProductName(),
        'db_version'     : dbMetaData.getDatabaseProductVersion(),
        'driver_name'    : dbMetaData.getDriverName(),
        'driver_version' : dbMetaData.getDriverVersion()
    ]
    manager.closeConnection()
    return map
}

//---------------------------------------------------------------------------
// Return list of schema maps for a DB
//---------------------------------------------------------------------------
def getSchemas( dbId, showSysTables ) {
    showSysTables = ( showSysTables == 'true' ? true : false )
    logger.INFO{ "getSchemas called for; ${dbId} (sys:${showSysTables})" }
    def mgr = Manager.create( dbId )
    def con = mgr.retrieveConnection()
    def list = []
    ResultSet schemas = con.getMetaData().getSchemas()
    while (schemas.next()) {
        String schema = schemas.getString("TABLE_SCHEM");
        if ( showSysTables || !(schema =~ "SYS.*" || schema =~ "NULLID" || schema =~ "SQLJ") ) {
            list << schema
        }
    }
    mgr.closeConnection()
    logger.INFO {"getSchemas: ${list}"}
    return list;    
}
    
//---------------------------------------------------------------------------
// Returns list of tables for a given schema
//---------------------------------------------------------------------------
def getTables(dbId, schema) {
    logger.INFO{ "getTables called for: ${dbId}::${schema}" }
    def tables = []
    def mgr = Manager.create( dbId )
    def con = mgr.retrieveConnection()
    ResultSet rs = con.getMetaData().getTables(null, schema, "%", null)
    while (rs.next()) {
        def table = rs.getString("TABLE_NAME")
        logger.INFO {"Got table: ${schema}.${table}" }
        tables << [
            'uid'       : "${schema}.${table}",
            'name'      : table,
            'type'      : 'table',
            'tableType' : rs.getString("TABLE_TYPE")
        ]
    }
    mgr.closeConnection()
    logger.INFO{ "${schema} tables: ${tables}" }
    return tables
}

//---------------------------------------------------------------------------
def getTableColumns( ds, table) {
    logger.INFO{ "called for: ${ds}::${table}" }
    def cols = []

    //-- Check DB vendor, and adjust SQL to get single record
    def sql
    def dbInfo = getDbInfo( ds )
    if ( dbInfo.db_name.startsWith("DB2") ) {
        sql = 'select * from '+table+' FETCH FIRST 2 ROWS ONLY'
    } else if ( dbInfo.db_name.startsWith("SQL") ) {   /// MS SQL*Server ??
        sql = 'select top 1 * from '+table
    } else if ( dbInfo.db_name.startsWith("Ora") ) {   /// MS SQL*Server ??
        sql = 'select * from '+table+' where rownum=1'
    } else {
        // Just grab it all and bail out
        sql = 'select * from '+table
    }

    Connection con = null
    def stmt = null

    try {
        def manager = Manager.create( ds )
        con = manager.retrieveConnection()
        stmt = con.createStatement()
        stmt.execute( sql )
        ResultSet rs = stmt.getResultSet()
        ResultSetMetaData rsMetaData = rs.getMetaData();
        
        for (int x = 1; x <= rsMetaData.getColumnCount(); x++) {
            cols << [
                name  : rsMetaData.getColumnLabel(x),
                type  : rsMetaData.getColumnTypeName(x)
            ]
        }
    } catch ( Exception e ) {
        logger.INFO {'Error on SQL processing' + e}
        cols = ["name":"ERROR", "type":"Error during SQL processing: ${e}"]
    } finally {
        try { stmt.close() } catch (Exception e) {}
        try { con.close()  } catch (Exception e) {}
    }
    logger.INFO{ "COLS: " + cols }
    return cols;
}

//---------------------------------------------------------------------------
def runSql( ds, sql) {
    def map = [:]
    Connection con = null
    def stmt = null

    try {
        def manager = Manager.create( ds )
        con = manager.retrieveConnection()
        stmt = con.createStatement()
        if ( stmt.execute( sql ) ) {
            def headers = [ ['name':'Row #', 'field':'_id_', 'type':'number' ] ]

            ResultSet rs = stmt.getResultSet()
            ResultSetMetaData rsMetaData = rs.getMetaData();
            
            for (int x = 1; x <= rsMetaData.getColumnCount(); x++) {
                def type
                switch (rsMetaData.getColumnType(x)) {
                case Types.BIGINT:
                case Types.INTEGER:
                case Types.NUMERIC:
                case Types.SMALLINT:
                case Types.TINYINT:
                case Types.DECIMAL:
                case Types.DOUBLE:
                case Types.FLOAT:
                    type = "number"
                    break;
                case Types.BINARY:
                case Types.BIT:
                case Types.BOOLEAN:
                    type = "boolean"
                    break;
                case Types.CHAR:
                case Types.LONGVARCHAR:
                case Types.VARCHAR:
                default:
                    type = "string"
                    break;
                }
                headers << [
                    name   : rsMetaData.getColumnLabel(x),
                    field  : 'C'+x, 'type':type
                ]
            }
            map.put( 'headers', headers )

            //-- Process result set
            def content = ['identifier':'_id_', 'label':'C1', 'items': [] ]
            def rowCntr = 1
            while (rs.next()) {
                def row = [ '_id_': rowCntr ]
                for( int c = 1; c <= rsMetaData.getColumnCount(); c++ ) {
                    def cellData = zero.util.XMLEncoder.escapeXML( rs.getString(c) );
                    row.put( [ ('C'+c), cellData ] )
                }
                content.items << row
                //-- Sanity check!
                if ( rowCntr++ >= 10000 ) {
                    break
                }
            }
            
            map.put('content', content)
            if ( rowCntr > 1 ) {
                map.put('message', "Successfully retrieved ${rowCntr} records")
            } else {
                map.put('message', "No records found")
            }
            map.put('messageType', 'Info')
            
        } else {
            //-- Assume an "update" (insert, update, delete) 
            def cnt = stmt.getUpdateCount()
            map.put('message', "${cnt} records processed.")
            map.put('messageType', 'Info')
        }

    } catch ( Exception e ) {
        logger.INFO {'processSql: Error on SQL processing' + e}
        map.put('message', "Error during SQL processing: ${e}")
        map.put('messageType', 'Error')

    } finally {
        try { stmt.close() } catch (Exception e) {}
        try { con.close()  } catch (Exception e) {}
    }
    return map
}

//---------------------------------------------------------------------------
// EOF: dba.groovy
//---------------------------------------------------------------------------
